@@ -1,4 +1,4 @@
-/* $Id: xfsm-util.c 4711 2004-11-01 16:10:55Z benny $ */
+/* $Id: xfsm-util.c 24158 2006-12-21 21:50:25Z benny $ */
 /*-
  * Copyright (c) 2003-2004 Benedikt Meurer <benny@xfce.org>
  * Copyright (c) 2004 Jasper Huijsmans <jasper@xfce.org>
@@ -203,17 +203,19 @@ void
 xfsm_window_add_border (GtkWindow *window)
 {
   GtkWidget *box1, *box2;
+  
+  gtk_widget_realize(GTK_WIDGET(window));
 
   box1 = gtk_event_box_new ();
   gtk_widget_modify_bg (box1, GTK_STATE_NORMAL, 
-                        &(box1->style->bg [GTK_STATE_SELECTED]));
+                        &(GTK_WIDGET(window)->style->bg [GTK_STATE_SELECTED]));
   gtk_widget_show (box1);
 
   box2 = gtk_event_box_new ();
   gtk_widget_show (box2);
   gtk_container_add (GTK_CONTAINER (box1), box2);
 
-  gtk_container_set_border_width (GTK_CONTAINER (box2), 3);
+  gtk_container_set_border_width (GTK_CONTAINER (box2), 6);
   gtk_widget_reparent (GTK_BIN (window)->child, box2);
 
   gtk_container_add (GTK_CONTAINER (window), box1);
