@@ -1,6 +1,6 @@
-/* $Id: xfsm-manager.c 20833 2006-04-09 18:08:17Z benny $ */
+/* $Id: xfsm-manager.c 23142 2006-09-13 15:52:01Z benny $ */
 /*-
- * Copyright (c) 2003-2004 Benedikt Meurer <benny@xfce.org>
+ * Copyright (c) 2003-2006 Benedikt Meurer <benny@xfce.org>
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -70,7 +70,6 @@
  */
 static gboolean   xfsm_manager_startup (void);
 static void       xfsm_manager_handle_failed (void);
-static void       xfsm_manager_startup_continue (const gchar *previous_id);
 static gboolean   xfsm_manager_startup_timedout (gpointer user_data);
 static void       xfsm_manager_load_settings (XfceRc *rc);
 static gboolean   xfsm_manager_load_session (void);
@@ -159,72 +158,6 @@ xfsm_manager_handle_failed (void)
 }
 
 
-static void
-xfsm_manager_startup_continue (const gchar *previous_id)
-{
-  gboolean startup_done = FALSE;
-  gchar buffer[1024];
-  XfceRc *rc;
-
-  xfsm_verbose ("Manager startup continues [Previous Id = %s]\n\n",
-                previous_id != NULL ? previous_id : "None");
-
-  if (startup_timeout_id != 0)
-    {
-      g_source_remove (startup_timeout_id);
-      startup_timeout_id = 0;
-
-      /* work around broken clients */
-      if (state != XFSM_MANAGER_STARTUP)
-        return;
-    }
-
-  startup_done = xfsm_startup_continue (previous_id);
-
-  if (startup_done)
-    {
-      xfsm_verbose ("Manager finished startup, entering IDLE mode now\n\n");
-      state = XFSM_MANAGER_IDLE;
-
-      if (!failsafe_mode)
-        {
-          /* handle apps that failed to start */
-          xfsm_manager_handle_failed ();
-
-          /* restore active workspace, this has to be done after the
-           * window manager is up, so we do it last.
-           */
-          g_snprintf (buffer, 1024, "Session: %s", session_name);
-          rc = xfce_rc_simple_open (session_file, TRUE);
-          xfce_rc_set_group (rc, buffer);
-          xfsm_manager_restore_active_workspace (rc);
-          xfce_rc_close (rc);
-
-          /* start legacy applications now */
-          xfsm_legacy_startup ();
-        }
-    }
-  else
-    {
-      startup_timeout_id = g_timeout_add (STARTUP_TIMEOUT,
-                                          xfsm_manager_startup_timedout,
-                                          NULL);
-    }
-}
-
-
-static gboolean
-xfsm_manager_startup_timedout (gpointer user_data)
-{
-  xfsm_verbose ("Manager startup timed out\n\n");
-
-  startup_timeout_id = 0; /* will be removed automagically once we return */
-  xfsm_manager_startup_continue (NULL);
-  
-  return FALSE;
-}
-
-
 static gboolean
 xfsm_manager_choose_session (XfceRc *rc)
 {
@@ -501,6 +434,72 @@ xfsm_manager_restart (void)
 }
 
 
+void
+xfsm_manager_startup_continue (const gchar *previous_id)
+{
+  gboolean startup_done = FALSE;
+  gchar buffer[1024];
+  XfceRc *rc;
+
+  xfsm_verbose ("Manager startup continues [Previous Id = %s]\n\n",
+                previous_id != NULL ? previous_id : "None");
+
+  if (startup_timeout_id != 0)
+    {
+      g_source_remove (startup_timeout_id);
+      startup_timeout_id = 0;
+
+      /* work around broken clients */
+      if (state != XFSM_MANAGER_STARTUP)
+        return;
+    }
+
+  startup_done = xfsm_startup_continue (previous_id);
+
+  if (startup_done)
+    {
+      xfsm_verbose ("Manager finished startup, entering IDLE mode now\n\n");
+      state = XFSM_MANAGER_IDLE;
+
+      if (!failsafe_mode)
+        {
+          /* handle apps that failed to start */
+          xfsm_manager_handle_failed ();
+
+          /* restore active workspace, this has to be done after the
+           * window manager is up, so we do it last.
+           */
+          g_snprintf (buffer, 1024, "Session: %s", session_name);
+          rc = xfce_rc_simple_open (session_file, TRUE);
+          xfce_rc_set_group (rc, buffer);
+          xfsm_manager_restore_active_workspace (rc);
+          xfce_rc_close (rc);
+
+          /* start legacy applications now */
+          xfsm_legacy_startup ();
+        }
+    }
+  else
+    {
+      startup_timeout_id = g_timeout_add (STARTUP_TIMEOUT,
+                                          xfsm_manager_startup_timedout,
+                                          NULL);
+    }
+}
+
+
+static gboolean
+xfsm_manager_startup_timedout (gpointer user_data)
+{
+  xfsm_verbose ("Manager startup timed out\n\n");
+
+  startup_timeout_id = 0; /* will be removed automagically once we return */
+  xfsm_manager_startup_continue (NULL);
+  
+  return FALSE;
+}
+
+
 gchar*
 xfsm_manager_generate_client_id (SmsConn sms_conn)
 {
@@ -979,6 +978,29 @@ xfsm_manager_close_connection (XfsmClient *client,
                 }
 #endif
             }
+
+          /* Run the SmDiscardCommand after the client exited in IDLE state.
+           * FIXME: Need to handle this better when we support restarting
+           * immediately properly.
+           * Unfortunately the spec isn't clear about the usage of the discard
+           * command. Have to check ksmserver/gnome-session, and come up with
+           * consistent behaviour.
+           * But for now, this work-around fixes the problem of the evergrowing
+           * number of xfwm4 session files when restarting xfwm4 within a session.
+           */
+          if (state == XFSM_MANAGER_IDLE && properties->discard_command != NULL)
+            {
+              xfsm_verbose ("Client Id = %s exited while in IDLE state, running "
+                            "discard command now.\n\n", properties->client_id);
+
+              g_spawn_sync (properties->current_directory,
+                            properties->discard_command,
+                            properties->environment,
+                            G_SPAWN_SEARCH_PATH,
+                            NULL, NULL,
+                            NULL, NULL,
+                            NULL, NULL);
+            }
         }
 
       running_clients = g_list_remove (running_clients, client);
